
# imports.
from classes.config import *
from apps.accounts.user import user
from apps.packages.packages import inc_package_manager

# the packages view.
class Packages(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, "packages/", "packages", website=website, landing_page=True)
	def view(self, request):

		# check authenticated.
		if not user.authenticated:
			return self.render(request, html="accounts/html/signin.html")
		elif not user.activated:
			return self.render(request, html="accounts/html/activate.html", template_data={
				"USER": user.info
			})
		else:

			# filters.
			filters, _ = self.parameters.get(request, {
				"os":"*",
				"price":"*",
			})

			# render.
			return self.render(request, {
				"USER": user.info,
				"PACKAGES":inc_package_manager.list(
					os=filters.os,
					price=filters.price,
				),
			})

		#

# the package view.
class Package(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, "packages/", "package", website=website)
	def view(self, request):

		# check authenticated.
		if not user.authenticated:
			return self.render(request, html="accounts/html/signin.html")
		elif not user.activated:
			return self.render(request, html="accounts/html/activate.html", template_data={
				"USER": user.info
			})
		else:

			# get params.
			parameters, response = self.parameters.get(request, [
				"package",
			])
			if not response.success:
				return self.error(message=response.error, request=request)

			# get package.
			try:
				package = inc_package_manager.list()[parameters.package]
			except KeyError:
				return self.error(message=f"Specified package [{parameters.package}] does not exist.", request=request)

			# render.
			installed, purchased, up_to_date = False, False, False
			response = inc_package_manager.installed(parameters.package)
			if response.success: installed = response.installed
			else: print("Error:",response.error)
			response = inc_package_manager.purchased(parameters.package)
			if response.success: purchased = response.purchased
			else: print("Error:",response.error)
			if installed:
				response = inc_package_manager.up_to_date(parameters.package)
				if response.success: up_to_date = response.up_to_date
				else: print("Error:",response.error)
			for key in ["category"]: package[key] = String(package[key]).capitalized_scentence()
			return self.render(request, {
				"OS":dev0s.defaults.vars.os,
				"USER": user.info,
				"PACKAGE":Dictionary(package)+{
					"purchased":purchased,
					"installed":installed,
					"up_to_date":up_to_date,
				},
			})

		#

# the package view.
class Docs(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, "packages/", "docs", website=website, except_xframe=True)
	def view(self, request):

		# check authenticated.
		if not user.authenticated:
			return self.render(request, html="accounts/html/signin.html")
		elif not user.activated:
			return self.render(request, html="accounts/html/activate.html", template_data={
				"USER": user.info
			})
		else:

			# get params.
			parameters, response = self.parameters.get(request, [
				"package",
			])
			if not response.success:
				return self.error(message=response.error, request=request)

			# get package.
			try:
				package = inc_package_manager.list()[parameters.package]
			except KeyError:
				return self.error(message=f"Specified package [{parameters.package}] does not exist.", request=request)

			# render.
			return self.render(request, {
				"USER": user.info,
				"PACKAGE":Dictionary(package)+{
				},
			})

		#

# the package view.
class Purchase(w3bsite.views.View):
	def __init__(self):
		w3bsite.views.View.__init__(self, "packages/", "purchase", website=website, except_xframe=True)
	def view(self, request):

		# check authenticated.
		if not user.authenticated:
			return self.render(request, html="accounts/html/signin.html")
		elif not user.activated:
			return self.render(request, html="accounts/html/activate.html", template_data={
				"USER": user.info
			})
		else:

			# get params.
			parameters, response = self.parameters.get(request, [
				"package",
			])
			if not response.success:
				return self.error(message=response.error, request=request)

			# get package.
			try:
				package = inc_package_manager.list()[parameters.package]
			except KeyError:
				return self.error(message=f"Specified package [{parameters.package}] does not exist.", request=request)

			# render.
			return self.render(request, {
				"USER": Dictionary(user.info)+{
					"api_key":user.api_key
				},
				"PACKAGE":Dictionary(package)+{
				},
			})

		#

# url patterns.
urlpatterns = w3bsite.views.build_urls([
	Packages(),
	Package(),
	Docs(),
	Purchase(),
])



# classes imports.
from classes.config import *
from apps.accounts.user import user
from apps.packages.packages import inc_package_manager

# sign in.
class SignIn(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/accounts/", "signin", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"username",
			"password",])
		if not response.success: return response

		# requests.
		response = user.signin(
			username=parameters.username,
			password=parameters.password,
		)
		if response.success:
			_response_ = inc_package_manager.connect()
			if not _response_.success: return _response_
		return response
		
		#

# sign in.
class SignOut(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/accounts/", "signout", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		return user.signout()
		
		#

# sign up.
class SignUp(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/accounts/", "signup", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"username",
			"email",
			"name",
			"password",
			"verify_password",
			#"code",
		])
		if not response.success: return response

		# requests.
		return user.signup(
			username=parameters.username,
			email=parameters.email,
			name=parameters.name,
			password=parameters.password,
			verify_password=parameters.verify_password,
			#code=parameters.code,
		)
		
		#

# request a reset password verification code.
class SendCode(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/accounts/", "send_code", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"username",
			"mode",])
		if not response.success: return response

		# requests.
		return user.send_code(
			username=parameters.username,
			mode=parameters.mode,
		)
		
		#

# reset the user's password.
class ResetPassword(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/accounts/", "reset_password", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"username",
			"code",
			"password",
			"verify_password",])
		if not response.success: return response

		# requests.
		return user.reset_password(
			username=parameters.username,
			code=parameters.code,
			password=parameters.password,
			verify_password=parameters.verify_password,
		)
		
		#

# activate a newly created user.
class Activate(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/accounts/", "activate", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"code",])
		if not response.success: return response

		# check authenticated.
		if not user.authenticated:
			return self.error("The current user is not authenticated.")

		# requests.
		return user.activate(
			code=parameters.code,
		)
		
		#
# edit user.
class Edit(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/accounts/", "edit", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"email",
			"name",])
		if not response.success: return response

		# check authenticated.
		if not user.authenticated:
			return self.error("The current user is not authenticated.")

		# requests.
		return user.edit(
			email=parameters.email,
			name=parameters.name,
		)
		
		#

# edit the user's password.
class EditPassword(w3bsite.views.Request):
	def __init__(self):
		w3bsite.views.Request.__init__(self, 
			"requests/accounts/", "edit_password", 
			website=website,
			auth_required=False, 
			root_required=False, )
	def request(self, request):
		
		# check authenicated.
		parameters, response = self.parameters.get(request, [
			"password",
			"new_password",
			"verify_password",])
		if not response.success: return response

		# check authenticated.
		if not user.authenticated:
			return self.error("The current user is not authenticated.")

		# requests.
		return user.edit_password(
			password=parameters.password,
			new_password=parameters.new_password,
			verify_password=parameters.verify_password,
		)
		
		#

# url patterns.
urlpatterns = w3bsite.views.build_urls([
	SignIn(),
	SignUp(),
	SignOut(),
	SendCode(),
	ResetPassword(),
	Activate(),
	Edit(),
	EditPassword(),
])

#